/* $Id$ */
package com.manageengine.servicedesk.notification.plugin;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.FileNotFoundException;

import java.net.URL;
import java.net.HttpURLConnection;

import java.text.SimpleDateFormat;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONException;

import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.jboss.security.Base64Encoder;
import com.manageengine.servicedesk.actionplugin.executor.*;

/**
 * @author Sivanand
 * Documentation: https://forums.manageengine.com/topic/external-action-plugin-feature-in-sdp
 * 
 */
public class xMattersPlugin extends DefaultActionInterface {

    // Location of the config file. 
    // ex: /etc/ManageEngine/ServiceDeskPlus-MSP/server/default/conf/plugins/xMatters
    private final String confFilePath = System.getProperty("jboss.home.dir") + File.separator + 
                                        "server" + File.separator + "default" + File.separator + 
                                        "conf" + File.separator + "plugins" + File.separator + 
                                        "xMatters" + File.separator + "xMattersConfig.xml"; // No i18n
                                        
    private long lastModifiedTime = 0L;

    private URL xMattersURL = null;
    private String authString = null;
    private String useMapping = "yes";
    private HashMap<String, String> userMappings = null;
    private final String dateFormat = "dd-MMM-yyyy HH:mm:ss"; // No i18n

    private static Logger logger = Logger.getLogger( xMattersPlugin.class.getName() );

    // Constructor.
    // Just load the XML config
    public xMattersPlugin() {

        try {
            loadXMLData();
        } catch(Exception x) {
            x.printStackTrace();
        }
    }

    /*
      INPUT JSON
     {
         "notificationid": "1111",
         "module": "Request",
         "moduleid": "1",
         "notificationtype": "req-assign",
         "notificationtime": "1386140462574",
         "userdetails": {
           "username": "administrator",
           "loginname": "administrator",
           "phone": "1234455",
           "mobile": "1234567890",
           "isuserloggedin": true
         }
     }
         
     OUTPUT JSON
     {
      "properties":{
        "EventID":"1111",
        "Module - Request / Task":"Request",
        "ModuleId - RequestID / TaskID":"1",
        "NotificationType":"req-assign",
        "NotificationTime":"12:31:02 04-Dec-2013"
      },
      "recipients":[
        {
           "targetName":"admin"
        }
      ]
     }
     */
    public JSONObject execute(ExecutorData executorData) throws Exception {

        JSONObject data = executorData.getDataJSON();
        JSONObject responseJSON = new JSONObject();
        
        logger.log(Level.INFO, "Received JSON data: " + data.toString());

        HttpURLConnection httpUrl = null;
        try {

            String payload = getxMattersPayload(data).toString();
            logger.log(Level.INFO, "xMatters JSON data: " + payload);

            httpUrl = (HttpURLConnection) xMattersURL.openConnection();
            httpUrl.setRequestMethod("POST"); // No i18n
            httpUrl.setUseCaches(false);
            httpUrl.setDoOutput(true);
            httpUrl.setRequestProperty("Authorization", "Basic " + authString); // No i18n
            httpUrl.setRequestProperty("Content-Type", "application/json; charset=UTF-8"); // No i18n
            //httpUrl.setRequestProperty("Content-Length", String.valueOf(payload.length())); // No i18n

            OutputStream out = null;
            BufferedReader in = null;

            try {
    
                httpUrl.connect();
                out = httpUrl.getOutputStream();
                out.write(payload.getBytes());
                out.flush();


                responseJSON.put("status", httpUrl.getResponseCode());
                responseJSON.put("message", httpUrl.getResponseMessage());
                
                in = new BufferedReader(new InputStreamReader(httpUrl.getInputStream()));

                StringBuilder response = new StringBuilder();
                String line;
                
                while((line = in.readLine()) != null) {
                    response.append(line);
                }
                logger.log(Level.INFO, "xMatters response: {0}", response.toString());
                responseJSON.put("response", response.toString());
            } catch( Exception e ) {
        
                logger.log( Level.INFO, "Exception: " + e );
            } finally {

                if(out != null) {
                    out.close();
                }
                if(in != null) {
                    in.close();
                }

            }
        } catch(IOException io) {
            
            if(httpUrl != null) {
            
                responseJSON.put("status", httpUrl.getResponseCode());
                responseJSON.put("message", httpUrl.getResponseMessage());
            }
            io.printStackTrace();
        } catch(JSONException json) {
            
            if(httpUrl != null) {
                
                responseJSON.put("status", httpUrl.getResponseCode());
                responseJSON.put("message", httpUrl.getResponseMessage());
            }
            json.printStackTrace();
        } catch( Exception e ) {
        
            if(httpUrl != null) {

                responseJSON.put("status", httpUrl.getResponseCode());
                responseJSON.put("message", httpUrl.getResponseMessage());
            }
 
           logger.log( Level.INFO, "Exception: " + e );
        }
        
        logger.log(Level.INFO, "Returning JSON data: {0}" + responseJSON.toString());
        return responseJSON;
    }

    private JSONObject getxMattersPayload(JSONObject data) throws JSONException {

        JSONObject returnValue = new JSONObject();

        JSONObject properties = new JSONObject();
        properties.put("EventID", data.getString("notificationid")); // No i18n
        properties.put("Module - Request / Task", data.getString("module")); // No i18n
        properties.put("ModuleId - RequestID / TaskID", data.getString("moduleid")); // No i18n
        properties.put("NotificationType", data.getString("notificationtype")); // No i18n

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(dateFormat);
        properties.put("NotificationTime", simpleDateFormat.format(Long.parseLong(data.getString("notificationtime")))); // No i18n

        JSONObject userData = data.getJSONObject("userdetails"); // No i18n
        JSONArray recipients = new JSONArray();
        JSONObject user = new JSONObject();
        
        // If we're using the mapping, then pull from userMappings,
        if( useMapping.equals( "no" ) ) {
          user.put("targetName", userData.get( "loginname" ) );
        }
        else {
          user.put("targetName", userMappings.get(userData.get("loginname"))); // No i18n
        }
        
        recipients.put(user);

        returnValue.put("properties", properties); // No i18n
        returnValue.put("recipients", recipients); // No i18n
        return returnValue;
    }

    
    /***
    * loadxMLData - pulls the endpoint, authentication and user mapping information from a flat xml file
    *   the location of this file is set confFilePath above. 
    */
    private void loadXMLData() throws Exception {

        // Load the file and initialize the root XML node. 
        Element root = getRootElement();

        // Extract the url value
        String urlString = root.getElementsByTagName("url").item(0).getTextContent(); // No i18n
        xMattersURL = new URL(urlString);

        // Extract the authentication information and encode in base64 for use later
        Element authElement = (Element) root.getElementsByTagName("auth").item(0); // No i18n
        StringBuilder authStr = new StringBuilder();
        authStr.append(authElement.getElementsByTagName("username").item(0).getTextContent()); // No i18n
        authStr.append(":");
        authStr.append(authElement.getElementsByTagName("password").item(0).getTextContent()); // No i18n
        authString = Base64Encoder.encode(authStr.toString());

        // Next, extract the user mapping. 
        Element userMappingsNode = (Element) root.getElementsByTagName("usermappings").item(0); // No i18n
        
        logger.log(Level.INFO, "Use mapping? " + userMappingsNode.getAttribute( "usemapping" ) );
        
        // Inspect the usemapping attribute to see if we should do the mapping from
        // SDP users to xM users
        // If no...
        if( userMappingsNode.getAttribute( "usemapping" ).equals( "no" ) ) {
        
           useMapping = "no";
        }
        
        // If yes... (or anything else)
        else {
          NodeList userNodes = userMappingsNode.getElementsByTagName("user"); // No i18n
          if(userNodes.getLength() > 0) {
              userMappings = new HashMap<String, String>();
          }
          for(int i = 0;i < userNodes.getLength();i++) {
              userMappings.put(((Element) userNodes.item(i)).getAttribute("sdpusername"), ((Element) userNodes.item(i)).getAttribute("xmattersusername")); // No i18n
          }
        }
    }

    /**
    * getRootElement - Read config file and return root XML node
    *  If the file has been changed read
    */
    private Element getRootElement() throws Exception {

        FileInputStream fileStream = null;
        try {

            lastModifiedTime = (new File(confFilePath)).lastModified();
            fileStream = new FileInputStream(confFilePath);

            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document document = builder.parse(fileStream);
            return document.getDocumentElement();
        } catch(FileNotFoundException fnf) {
            fnf.printStackTrace();
        } finally {
            if(fileStream != null) {
                fileStream.close();
            }
        }
        return null;
    }

    public void reloadConfig() {

        if((new File(confFilePath)).lastModified() > lastModifiedTime) {

            try {
                loadXMLData();
            } catch(Exception x) {
                x.printStackTrace();
            }
        }
    }

  
}


